package PureMessage::LogReader;

use strict;
use base 'Exporter';
our @EXPORT_OK = qw(
);

sub _unpack_resume_info {
    my $self = shift;
    my $resume_info = shift;
    $resume_info = "0," unless defined $resume_info;
    ($self->{lastpos}, $self->{lastline}) = split /,/, $resume_info, 2;
}

sub _pack_resume_info {
    my $self = shift;
    return "$self->{lastpos},$self->{lastline}";
}

sub _set_record_seperator {
    my $fh = shift;
    select((select($fh), $/="\n")[0]);
}

sub _resume {
    my ($self, $fh) = @_;
    seek($fh, $self->{lastpos}, 0); # SEEK_SET is 0
    my $line = <$fh>;
    return $line && $self->{lastline} eq $line;
}

sub _open_rotated {
    my $self = shift;
    my $rotated_name = $self->{rotate_sub}($self->{file});
    if (-e $rotated_name) {
	if (open(my $fh, $rotated_name)) {
	    _set_record_seperator($fh); # just in case
	    if ($self->_resume($fh)) {
		$self->{next_handle} = $self->{handle};
		$self->{handle} = $fh;
		return 1;
	    }
	}
    }
    return 0;
}

sub _open {
    my $self = shift;
    open ($self->{handle}, $self->{file}) || die "Can't open logfile: $self->{file}";
    _set_record_seperator($self->{handle}); # just in case
    if (!$self->_resume($self->{handle})) {
	seek($self->{handle}, 0, 0); #  SEEK_SET is 0
	unless ($self->_open_rotated) {
	    $self->{lastpos} = tell($self->{handle});
	    $self->{lastline} = '';
	}
    }
}

sub new {
    my $cls = shift;
    my $self = {};
    bless $self, $cls;
    my %params = @_;
    $self->{file} = $params{-logfile} || die "No logfile specified";
    $self->{lastpos} = 0;
    $self->{lastline} = '';
    if ($params{-rotate_sub}) { 
	$self->{rotate_sub} = $params{-rotate_sub};
    }
    else {
	$self->{rotate_sub} = sub {
	    my $fn = shift;
	    $fn =~ s/(.*)/$1.1/;
	    return $fn;
	};
    }
    $self->_unpack_resume_info($params{-resume_info});
    $self->_open;
    return $self;
}

sub readline {
    my $self = shift;
    my $fh = $self->{handle};
    my $pos = tell($fh);
    my $line = <$fh>;
    if (!$line) {
	if ($self->{next_handle}) {
	    close $self->{handle};
	    $self->{handle} = delete $self->{next_handle};
	    $self->{lastpos} = 0;
	    $self->{lastline} = '';
	    return $self->readline;
	}
	return undef;
    }
    else {
	$self->{lastpos} = $pos;
	$self->{lastline} = $line;
	return $line;
    }
}

sub resume_info {
    return $_[0]->_pack_resume_info;
}

1;

=head1 NAME

PureMessage::LogReader - Read a log file, supports resuming 

=head1 SYNOPSIS

 use PureMessage::LogReader;

 my $logreader = PureMessage::LogReader->new(
     -logfile => "my.log",
 );

 while (my $line = $logreader->readline) {
     # parse line
 }

 my $resume_info = $logreader->resume_info;

or

 use PureMessage::LogReader;

 my $resume_info = ...;

 my $logreader = PureMessage::LogReader->new(
     -logfile => "my.log",
     -resume_info => $resume_info,
 );
 while (my $line = $logreader->readline) {
     # parse line
 }

 $resume_info = $logreader->resume_info;

=head1 DESCRIPTION

=over

=item $logreader = PureMessage::LogReader->new(-logfile=>"logfile");

Returns a LogReader object that can be used to read logfiles. Options
supported include:

=over

=item -logfile

Base log file name.

=item -resume_info

Information returned by previous log file parsing session, used to resume
parsing the log file at the same position we left off last time.

=item -rotate_sub

References to a subroutine that transforms -logfile into the filename it would
have been log rotated to. [Note: compressed log rotation will not work.] The 
default subroutine adds ".1" to the -logfile parameter.

=back

=item $logreader->readline

Returns the next line in the -logfile or undef if the end of the log file is
reached. In the event a log rotate file reaches its end readline will move on
to the current -logfile automatically.

=item $logreader->resume_info

Returns a string that can be used to resume parseing a -logfile by passing it
to ->new via the -resume_info parameter.

=back

=head1 TODO

Support compressed rotated files
